// DIB.cpp: implementation of the CDIB class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DIB.h"

#define Convert565( r, g, b ) ( ((r>>3)<<11) | ((g>>2)<<5) | (b>>3) )
#define Convert555( r, g, b ) ( ((r>>3)<<10) | ((g>>3)<<5) | (b>>3) )

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDIB::CDIB(char *FileName)
: m_pBmi  ( NULL ), m_ImgSize ( 0 ),
  m_pImage( NULL ), m_InfoSize( 0 ),
  m_pBmih ( NULL ), m_bInstalled( FALSE )
{
	if( FileName != NULL )
	{
		if( !Load( FileName ) )
		{
			if( m_pBmi )
			{
				delete [] (BYTE *)m_pBmi;
				m_pBmi  = NULL;
				m_pBmih = NULL;
			}

			if( m_pImage )
			{
				delete [] (BYTE *)m_pImage;
				m_pImage = NULL;
			}

			return;
		}
	}
}

CDIB::~CDIB()
{
	RemoveAll();	
}

BOOL CDIB::Load(char *FileName)
{
	HANDLE handle;
	BITMAPFILEHEADER bmfh;
	DWORD ReadnSize;

	handle = CreateFile( FileName, GENERIC_READ, NULL, NULL, OPEN_EXISTING,
						 FILE_ATTRIBUTE_NORMAL, NULL);
	if( handle == INVALID_HANDLE_VALUE )
	{
		return FALSE;
	}

	ReadFile( handle, &bmfh, sizeof( bmfh ), &ReadnSize, NULL );

	if( bmfh.bfType != *(WORD *)"BM" )  
	{
		return FALSE;
	}

	m_InfoSize = bmfh.bfOffBits - sizeof( bmfh );
	m_pBmi  = (BITMAPINFO *) new BYTE[ m_InfoSize ];
	m_pBmih = (BITMAPINFOHEADER *)m_pBmi;

	ReadFile( handle, m_pBmi, m_InfoSize, &ReadnSize, NULL );

	if( m_pBmih->biBitCount < 8 )
	{
		return FALSE;
	}

	m_ImgSize = GetBytePerLine() * GetABSHeight();   //abs(GetHeight());
	m_pImage  = new BYTE[ m_ImgSize ];
	ReadFile( handle, m_pImage, m_ImgSize, &ReadnSize, NULL );
	CloseHandle( handle );

	if( GetHeight() < 0 )
	{
		BYTE *image = new BYTE[m_ImgSize];
		BYTE *pDest = image;
		BYTE *pSrc  = m_pImage + m_ImgSize - GetBytePerLine();
		// BYTE *pSrc  = m_pImage + (GetABSHeight() - 1) * GetBytePerLine();
		LONG i, y = GetABSHeight();
		LONG x = GetBytePerLine();

		for( i = 0; i < y; i++ )
		{
			CopyMemory( pDest, pSrc, x );
			pDest += x;
			pSrc  -= x;
		}

		CopyMemory( m_pImage, image, m_ImgSize );
		delete [] image;
	}

	m_bInstalled = TRUE;
	return TRUE;
}

LONG CDIB::GetBytePerLine()
{
	LONG xLen = m_pBmih->biWidth;
	xLen = xLen * ( GetPixelByteCount() );
	xLen = (xLen + 3) & ~3;
	return xLen;
}

BYTE CDIB::GetPixelByteCount()
{
	return( m_pBmih->biBitCount / 8 );
}

LONG CDIB::GetHeight()
{
	return m_pBmih->biHeight;
}

LONG CDIB::GetWidth()
{
	return m_pBmih->biWidth;
}

LONG CDIB::GetABSHeight()
{
	return abs( m_pBmih->biHeight );
}

HBITMAP CDIB::CreateBitmap()
{
	HDC hdc = GetDC( NULL );
	HBITMAP hBitmap;

	hBitmap = CreateDIBitmap( hdc, m_pBmih, CBM_INIT, m_pImage, m_pBmi, DIB_RGB_COLORS );
	ReleaseDC( NULL, hdc );

	return hBitmap;
}

CDIB::CDIB(CDIB &r)
: m_pBmi  ( NULL ), m_ImgSize ( r.m_ImgSize  ),
  m_pBmih ( NULL ), m_InfoSize( r.m_InfoSize ),
  m_pImage( NULL ), m_bInstalled( FALSE ) 
{
	if( m_InfoSize )
	{
		m_pBmi  = (BITMAPINFO *) new BYTE[ m_InfoSize ];
		m_pBmih = (BITMAPINFOHEADER *) m_pBmi;
		
		CopyMemory( m_pBmi, r.m_pBmi, m_InfoSize );
	}

	if( m_ImgSize )
	{
		m_pImage = new BYTE[ m_ImgSize ];
		CopyMemory( m_pImage, r.m_pImage, m_ImgSize );
		m_bInstalled = TRUE;
	}
}

CDIB& CDIB::operator=(CDIB &r)
{
	if( this == &r )
	{
		return *this;
	}

	RemoveAll();
	m_bInstalled = r.m_bInstalled;

	if( m_bInstalled )
	{
		m_pBmi  = (BITMAPINFO *) new BYTE[m_InfoSize];
		m_pBmih = (BITMAPINFOHEADER *)m_pBmi;
		CopyMemory( m_pBmi, r.m_pBmi, m_InfoSize );

		m_pImage = new BYTE[m_ImgSize];
		CopyMemory( m_pImage, r.m_pImage, m_ImgSize );
	}

	return *this;
}

void CDIB::RemoveAll()
{
	if( m_pBmi )
	{
		delete [] (BYTE *)m_pBmi;
		m_pBmi  = NULL;
		m_pBmih = NULL;
	}

	if( m_pImage )
	{
		delete [] (BYTE *)m_pImage;
		m_pImage = NULL;
	}

	m_ImgSize = m_InfoSize = 0;
	m_bInstalled = FALSE;
}


void CDIB::Convert16(char *FileName, BOOL flag)
{
	switch( m_pBmih->biBitCount )
	{
		case 8:                            // 256Į϶..
			Convert256To16( FileName, flag );
			break;
		case 24:                           //24 Ʈ Į϶..
			Convert24To16( FileName, flag );
			break;
		default:
			OutputDebugString( "16Ʈ Ʈ ȵ!!\n" );
			break;
	}
}

void CDIB::Convert16(BOOL flag)
{
}

void CDIB::Convert256To16(char *FileName, BOOL flag)
{
/*
	BITMAPFILEHEADER Bmfh;
	ZeroMemory(&Bmfh,sizeof(bmfh));

	Bmfh.bfType = *(WORD *)"BM";
	Bmfh.bfOffBits = sizeof(Bmfh) + m_InfoSize;

	BITMAPINFOHEADER Bmih;

	Bmih.biSize = sizeof(Bmih);
	Bmih.biWidth = m_pBmih->biWidth;
	Bmih.biHeight = m_pBmih->biHeight;
	Bmih.biPlanes = 1;
	Bmih.biBitCount = 16;

	LONG xlen = (m_pBmi->bmiHeader.biWidth) * 2;
	xlen = (xlen+3) & ~3;
	Bmih.biSizeImage = xlen * m_pBmih->biHeight;
	
	RGBQUAD *pRgb;
/**/

	BITMAPFILEHEADER Bmfh;
	
	ZeroMemory( &Bmfh, sizeof( Bmfh ) );
	
	Bmfh.bfType = *(WORD *)"BM";   // 0x4d42
	Bmfh.bfOffBits = sizeof( Bmfh ) + sizeof( BITMAPINFOHEADER ); // ش + ش   (16Ʈ RGB尡 Ƿ..)
	
	BITMAPINFO *pBmi = (BITMAPINFO *) new BYTE[ sizeof( BITMAPINFOHEADER ) ];
	CopyMemory( pBmi, m_pBmi, sizeof( BITMAPINFOHEADER ) );
	
	pBmi->bmiHeader.biBitCount = 16;
	
	LONG xlen = ( pBmi->bmiHeader.biWidth ) * 2;	// biWidth ȼ Ѵ.  xlen   뷮 . ׷Ƿ ϱ2 (16Ʈ) (1ȼ 2Ʈ)
	xlen = (xlen + 3) & ~3;							// 4ǹ 
	pBmi->bmiHeader.biSizeImage = xlen * pBmi->bmiHeader.biHeight;
	
	Bmfh.bfSize = sizeof( Bmfh ) + m_InfoSize + pBmi->bmiHeader.biSizeImage; //  + ش + ̹ũ
	
	WORD *pImage = new WORD[ pBmi->bmiHeader.biSizeImage / 2 ];   ////
	
	LONG i, j;
	LONG x = pBmi->bmiHeader.biWidth;
	LONG y = pBmi->bmiHeader.biHeight;
	
	BYTE *pSrc  = m_pImage;
	WORD *pDest = pImage;
	BYTE r,g,b;
	
	for( j = 0; j < y; j++ )
	{
		for( i = 0; i < x; i++ )
		{
			b = m_pBmi->bmiColors[ pSrc[i] ].rgbBlue;   // BITMAPINFO  RGBQUADü(BmiColors) Blue . 1Ʈ
			g = m_pBmi->bmiColors[ pSrc[i] ].rgbGreen;  // BITMAPINFO  RGBQUADü(BmiColors) Green . 1Ʈ     
			r = m_pBmi->bmiColors[ pSrc[i] ].rgbRed;    // BITMAPINFO  RGBQUADü(BmiColors) Red . 1Ʈ
			
			if( flag )    // 565  .
			{
				pDest[i] = Convert565( r, g, b );   // pDest WORD 2Ʈ.. 
			}										//    3Ʈ  
													// Ʈ Ͽ ڸ DWORD  pDest[i] OR 
			else         //  555  .
			{
				pDest[i] = Convert555( r, g, b );
			}
		}
		pSrc  += GetBytePerLine();
		pDest += (xlen / 2);
	}
	
	DWORD WritenSize;
	HANDLE handle = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL, NULL );
	WriteFile( handle, &Bmfh,  sizeof( Bmfh ),              &WritenSize, NULL );
	WriteFile( handle, pBmi,   sizeof( BITMAPINFOHEADER ),  &WritenSize, NULL );
	WriteFile( handle, pImage, pBmi->bmiHeader.biSizeImage, &WritenSize, NULL );
	
	CloseHandle(handle);
	
	delete [] (BYTE *) pBmi;
	delete [] pImage;
}

void CDIB::Convert24To16(char *FileName, BOOL flag)
{
	BITMAPFILEHEADER Bmfh;

	ZeroMemory( &Bmfh, sizeof( Bmfh ) );

	Bmfh.bfType = *(WORD *)"BM";   // 0x4d42
	Bmfh.bfOffBits = sizeof( Bmfh ) + m_InfoSize;

	BITMAPINFO *pBmi = (BITMAPINFO *) new BYTE[ m_InfoSize ];
	CopyMemory( pBmi, m_pBmi, m_InfoSize );

	pBmi->bmiHeader.biBitCount = 16;

	LONG xlen = ( pBmi->bmiHeader.biWidth ) * 2;    //biWidth ȼ Ѵ.  xlen   뷮 . ׷Ƿ ϱ2 (16Ʈ) (1ȼ 2Ʈ)
	xlen = (xlen + 3) & ~3;							// 4ǹ 
	pBmi->bmiHeader.biSizeImage = xlen * pBmi->bmiHeader.biHeight;

	Bmfh.bfSize = sizeof(Bmfh) + m_InfoSize + pBmi->bmiHeader.biSizeImage; //  + ش + ̹ũ

	WORD *pImage = new WORD[ pBmi->bmiHeader.biSizeImage / 2 ];   ////
	
	LONG i, j;
	LONG x = pBmi->bmiHeader.biWidth;
	LONG y = pBmi->bmiHeader.biHeight;
	
	BYTE *pSrc  = m_pImage;
	WORD *pDest = pImage;
	BYTE r, g, b;

	for( j = 0; j < y; j++ )
	{
		for( i = 0; i < x; i++ )
		{
			b = pSrc[ i * 3     ];		// Ʈ Ǿֱ⶧ b,g,r
			g = pSrc[ i * 3 + 1 ];		//  ȼ b,g,r  Ѵ. 1Ʈ(8Ʈ)
			r = pSrc[ i * 3 + 2 ];

			if( flag )		// 565  .
			{
				pDest[i] = Convert565( r, g, b );   // pDest WORD 2Ʈ.. 
			}										//    3Ʈ  
													// Ʈ Ͽ ڸ DWORD  pDest[i] OR 
			else			//  555  .
			{
				pDest[i] = Convert555( r, g, b );
			}
		}
		pSrc  += GetBytePerLine();
		pDest += (xlen / 2);
	}

	DWORD WritenSize;
	HANDLE handle = CreateFile( FileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL, NULL );
	WriteFile( handle, &Bmfh,  sizeof(Bmfh),				&WritenSize, NULL );
	WriteFile( handle, pBmi,   m_InfoSize,					&WritenSize, NULL );
	WriteFile( handle, pImage, pBmi->bmiHeader.biSizeImage, &WritenSize, NULL );

	CloseHandle( handle );

	delete [] (BYTE *) pBmi;
	delete [] pImage;
}

BYTE * CDIB::GetImage()
{
	return m_pImage;
}

DWORD CDIB::GetImageSize()
{
	return m_ImgSize;
}
